/**
 * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.crashnote.core.collect.impl.excp;

import com.crashnote.core.collect.BaseCollector;
import com.crashnote.core.config.*;
import com.crashnote.core.model.data.*;

/**
 * Collector to transform exception information (e.g. file, class, method) into a structured form.
 */
public class ExcpStackCollector<C extends Config>
    extends BaseCollector<C> {

    // SETUP ======================================================================================

    public ExcpStackCollector(final C config) {
        super(config);
    }

    // INTERFACE ==================================================================================

    public DataObject collectTrace(final String excpMsg, final String excpCls, final StackTraceElement[] stackTrace) {
        final DataObject excp = newExcp(excpMsg, excpCls);
        {
            final DataArray trace = createDataArr();
            for (final StackTraceElement element : stackTrace) {
                final int line = element.getLineNumber();
                final String meth = element.getMethodName();
                final String file = element.getFileName();
                final String cls = element.getClassName();
                final StringBuilder sb = new StringBuilder(cls)
                    .append(':').append(file)
                    .append(":").append(meth)
                    .append(":").append(line);
                trace.add(sb.toString());
            }
            excp.put("trace", trace);
            //excp.put("lines", stackTrace.length);
        }
        return excp;
    }

    // SHARED =====================================================================================

    DataObject newExcp(final String msg, final String cls) {
        final DataObject excp = createDataObj();
        excp.put("msg", simplifyMessage(msg));
        excp.put("cls", cls.replaceFirst("java\\.lang\\.", ""));
        return excp;
    }

    // INTERNALS ==================================================================================

    private String simplifyMessage(final String msg) {
        if (msg == null) return msg;
        else {
            final int idx = msg.indexOf("; nested exception is");
            if (idx != -1) {
                return msg.substring(0, idx);
            } else return msg;
        }
    }

}
